#include "map.h"
#include "resourcemanager.h"

#include <GL/gl.h>
#include <cstring>
#include <fstream>

using namespace std;

void Map::renderColumn(int x, const ResourceManager &rm)
{
	if ((x < 0) || (x >= MapWidth)) return;
	// Render tiles
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	rm.tiletexture.Bind();
	
	glColor3f(blocks[x].colour.r,
		blocks[x].colour.g,
		blocks[x].colour.b);
	float leftx = (float) x;
	for (int y = 0; y< MapHeight; y++) {
		MapBlock & block = blockAt(x,y);
		if (block.type == MapBlock::GroundBlock) {
			float bottomy = (float) y;
			glBegin(GL_QUADS);
			glTexCoord2f(0,0);
			glVertex3f(leftx, bottomy, 0);
			glTexCoord2f(1,0);
			glVertex3f(leftx+1,bottomy,0);
			glTexCoord2f(1,1);
			glVertex3f(leftx+1,bottomy+1,0);
			glTexCoord2f(0,1);
			glVertex3f(leftx,bottomy+1,0);
			glEnd();
		}
	}
	// Render towers
	glColor3f(1,1,1);
	glEnable(GL_LIGHTING);
	rm.towertexture.Bind();

	float towerscale = 1.0 / 4.0;
	for (int y = 0; y< MapHeight; y++) {
		MapBlock & block = blockAt(x,y);
		if (block.type == MapBlock::TowerBlock) {
			float midy = (float) y + 0.5;
			float midx = (float) x + 0.5;
			glPushMatrix();
			glTranslatef(midx, midy, 0);
			glScalef(towerscale,towerscale,towerscale);
			rm.towermodel.draw(true);
			glPopMatrix();
		}
		
	}
}

void Map::init(int level)
{
	char filename[30];
	sprintf(filename,"levels/level%d.txt", level);
	ifstream in(filename);
	if (! in.good()) {
		throw "Unable to open level file";
	}
	
	int x = 1;
	while (! in.eof()) {
		char buf[100];
		in.getline(buf,100);
		int len = std::strlen(buf);
		for (int y=0; y<len; y++) {
			MapBlock & block = blockAt(x,y);
			block.setFromChar(buf[y]);
		}
		if (len >= 13) {
			blocks[x].colour.r = (float) (buf[10] - '0') / 9.0;
			blocks[x].colour.g = (float) (buf[11] - '0') / 9.0;
			blocks[x].colour.b = (float) (buf[12] - '0') / 9.0;
		} else {
			blocks[x].colour = blocks[x-1].colour;
		}
		if (len >= 15) {
			blocks[x].speed = (int) (buf[14] - '0');
		} else {
			blocks[x].speed = blocks[x-1].speed;
		}
		x++;
	}
}

